# @command VRML
# @title   Einbinden von VRML-Dateien
# @index   
# @syntax  \#VRML\{optionen\}
# @desc    Mit Hilfe des {\tt \#VRML}-Befehls wird eine VRML-Datei
#          ({\tt .wrl}) ins Dokument eingebunden. In der 
#          \LaTeXe-Version wird, falls vorhanden, ein alternatives
#          Bild dargestellt. Beim Generieren der HTML-Version
#          (mit {\tt [-copy]}-Schalter) wird die VRML-Datei vom
#          Konverter gelesen, und alle in dieser Datei benutzten
#          Quellen ({\tt url \dq quelle\dq}) werden in das Output-Verzeichnis
#          mitkopiert.\\
# @example_begin
#  #VRML{ file=VRML/m2t.wrl,        % VRML-Datei
#      embed,hsign="Ein Quader",    % Eingebettet mit Unterschrift
#      alt=Gif/vrml_sc.gif,         % Alternatives Bild
#      width=10cm,                  % Bildgroesse
#      center,                      % Zentriert
#      sign="Ein Screenshot",       % Bildunterschrift
#      hwidth=80\\%,                % HTML-Breite
#      hheight=80\\%                % HTML-hoehe
#  }
# @example_end
#
#   @optname file
#   @optargs <vrml-datei>
#       Diese Option ist notwendig. {\tt vrml-datei} wird in die HTML-Version integriert.
#       \\Beispiel: {\tt file=VRML/space.wrl} 
#
#   @optname alt
#   @optargs <alternativbilddatei>
#       Die Bilddatei wird in der \LaTeXe-Version eingef"ugt.
#       Die {\tt alternativbilddatei} mu"s den gesamten (relativen oder absoluten) Pfad enthalten. 
#
#   @optname (no)center
#       Siehe Seite \pageref{cmd:IMAGE:(no)center} ({\tt \#IMAGE}-Optionen). 
#
#   @optname codelink
#   @optargs \dq linkname\dq
#       Es wird ein Link mit dem Namen {\tt linkname} auf eine HTML-Datei,
#       die den Quellkode von der VRML-Datei enth"alt, erzeugt.
#       Diese HTML-Datei wird automatisch vom Master-Konverter generiert und hat das gleiche Layout wie
#       das ganze Dokument. Die Datei wird in dem Unterverzeichnis {\tt \$OUTPUT\_DIR/Sources} abgelegt.
#       \\Defaultwert: {\tt \dq Source\dq} 
#
#   @optname (no)embed
#       Die {\tt vrml-datei} (nicht) einbetten. Bei der Option {\tt noembed} wird eine Verkn"upfung  
#       mit dem Namen {\tt hsign} auf die Datei erstellt. 
#
#   @optname (no)figure
#       Siehe Seite \pageref{cmd:IMAGE:(no)figure} ({\tt \#IMAGE}-Optionen). 
#
#   @optname height
#   @optargs <bildh"ohe>
#       Siehe Seite \pageref{cmd:IMAGE:height} ({\tt \#IMAGE}-Optionen). 
#
#   @optname hheight
#   @optargs <html-h"ohe>
#       HTML-H"ohe in Pixel oder Prozent.
#       \\Defaultwert: 100\% 
#
#   @optname hsign
#   @optargs \dq html-unterschrift\dq
#       Bei eingebetteter {\tt vrml-datei} Unterschrift, sonst Verkn"upfungsname.
#       Diese Option kann auch die HTML-Tags enthalten.
#       \\Beispiel: {\tt hsign= \dq Eine <I>Space</I>-Welt\dq}
#
#   @optname hwidth
#   @optargs <html-breite>
#       HTML-Breite in Pixel oder Prozent.
#       \\Defaultwert: 100\%  
#
#   @optname label
#   @optargs <labelbezeichner>
#       Siehe Seite \pageref{cmd:IMAGE:label} ({\tt \#IMAGE}-Optionen). 
#
#   @optname options
#   @optargs \dq EMBED-Optionen\dq
#       Hier kann man einige Optionen f"ur den {\tt <EMBED>}-Tag
#       eingeben.
#
#   @optname sign
#   @optargs \dq bildunterschrift\dq
#       Siehe Seite \pageref{cmd:IMAGE:sign} ({\tt \#IMAGE}-Optionen). 
#
#   @optname width
#   @optargs <bildbreite>
#       Siehe Seite \pageref{cmd:IMAGE:width} ({\tt \#IMAGE}-Optionen). 
#
# @end

# VRML-Modul
sub vrml_work {
  package vrml;

  my ($wert,$flag);
  local ($use_name,$use_alt_name,$use_sign,$use_imglink);
  local ($use_local_name,$use_type);
  local ($use_relativ_name);
  local ($use_dir);
  local ($use_hheight,$use_hwidth);
  local $use_codelink;
  local ($use_label);
  local ($noticy_key);
  local ($use_embed,$use_hsign);
  local $use_embed = $main::USE_EMBED;
  local $use_center = $main::USE_CENTER;
  local ($hsign_flag,$codelink_flag);
  local ($mtime);
  local $status;          # gleich null, wenn die
                  # Konvertierung fehlerfrei
  local (@vrml_source);      # speichert die fuer eine VRML-Datei
                  # zugehoerigen Quellen (.mid,.jpg ...)
  my (@stat_liste);
  my $file_name;          # Dateiname, der angegeben ist
                  # wird in die NTC-Datei eingetragen
  my $report_state = $main::REPORT_WRITE; # merkt altes Report-Zustand

  local $COMMAND=shift @_;
  local (@TO_IMAGE);        # Liste zur Bildbearbeitung
  local (@PARAMS) = @_;      # uebergebene Liste
  
  # wenn keine Erlaubung zum Schreiben, dann Ausstieg
  unless ($main::TEXFILE_WRITE_OK) {return;}
  
  # Flags setzen
  foreach $term (@PARAMS) {    
    ($flag,$wert)=&main::my_quotewords("=",0,$term);
    $flag =~ s/(\s*)//g;      # Trennzeichen loeschen
    SWITCH: {
      if ($flag eq "file") {$use_name=$wert; $file_name=$wert; last SWITCH;}
      if ($flag eq "embed") {$use_embed="1"; last SWITCH;}
      if ($flag eq "imglink") {$use_imglink="1"; last SWITCH;}
      if ($flag eq "noembed") {$use_embed=""; last SWITCH;}
      if ($flag eq "hsign") { $use_hsign=$wert; $hsign_flag="1"; last SWITCH;}
      if ($flag eq "hwidth") { $use_hwidth=$wert; last SWITCH;}
      if ($flag eq "hheight") { $use_hheight=$wert; last SWITCH;}
      if ($flag eq "sign") { $use_sign=$wert; last SWITCH;}
      if ($flag eq "codelink") { $use_codelink=$wert; $codelink_flag=1; last SWITCH;}

      if ($flag eq "alt") { $use_alt_name=$wert; push @TO_IMAGE,"file=$wert"; last SWITCH;}
      if ($flag eq "label") { $use_label=$wert; push @TO_IMAGE,"label=$wert"; last SWITCH;}
      if ($flag eq "center") { $use_center=1; push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nocenter") { $use_center=""; push @TO_IMAGE,"$flag"; last SWITCH;}

      if ($flag eq "figure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nofigure") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "nospace") { push @TO_IMAGE,"$flag"; last SWITCH;}
      if ($flag eq "width") { push @TO_IMAGE,"width=$wert"; last SWITCH;}
      if ($flag eq "height") { push @TO_IMAGE,"height=$wert"; last SWITCH;}
      if ($flag eq "options") { $use_options=$wert; last SWITCH;}

      &main::my_exit("Unknown flag in $COMMAND : $flag\n");
    }
  }
  
  # wenn es nur um das Bild geht
  if (($main::OUTPUTFORMAT eq "latex") || ($main::OUTPUTFORMAT eq "pdf")) {
    if ($use_alt_name) {
      push @TO_IMAGE,"sign=$use_sign"; 
      unshift @TO_IMAGE,"$COMMAND";
      &main::image_work(@TO_IMAGE);
    } else { &main::my_print("$COMMAND : Not Alt-File\n"); }
    return;
  } else {
    &main::my_print("$COMMAND (l.$main::INPUTFILE_LINE) : File=$use_name\n");
  }


  &to_flag_check;    # Syntax pruefen und wenn noetig die
            # default-Werte setzen
  
  $use_relativ_name = &main::get_relativ_name($use_name);  # relativer Pfadname
  $use_local_name = &main::get_base_name($use_name);  # Basisname
  $use_type = &main::get_file_type($use_name);    # Dateierweiterung
  &main::statistic_count($use_type);          # fuer Statistik
  # Absoluten Pfad erkuendigen
  $use_dir=&main::get_abs_pfad($use_name);
  $use_name = "$use_dir/$use_local_name.$use_type";

  # den Notice-Schluessel bilden
  $noticy_key=&main::build_key($main::OUTPUTFORMAT,$file_name);

  # Dateieigenschaften erkundigen (Dateigroesse und die letzte
  # Modifikationszeit)
  @stat_liste = stat $use_name;
  $mtime = $stat_liste[9];

  # Schreiben in die Report-Version erlauben
  $main::REPORT_WRITE = 1;
  # Entsprechende Mechanismen aufrufen
  &html_work;
  # Schreiben in die Report-Version zuruecksetzen
  $main::REPORT_WRITE = $report_state;
  
  # wenn alles geklappt, dann Notice-Eintrag machen
  if (!($status)) { #wenn erfolgreich dann eintragen
    $main::NOTICE_NEW{$noticy_key}="$mtime";
  }

  return;

  # Prueft Syntax und Existenz von Quellen vom Master-Befehl 
  sub to_flag_check {
    unless ($use_name) {&main::my_exit("File is not statement.\n");}
    unless (-f $use_name) {&main::my_msg("File '$use_name' not found.\n");}
  
    # Pruefe die HTML-Groessen
    $use_hwidth =~ s/(\s+)//g;
    if ($use_hwidth) { 
      unless ($use_hwidth =~ /^((\d+)|(\d+%))$/) {
        &main::my_msg("Not allowed value in $COMMAND for hwidth : $use_hwidth\n");
        $use_hwidth="100%";
      }
    } else {
      $use_hwidth="100%";
    }
    $use_hheight =~ s/(\s+)//g;
    if ($use_hheight) { 
      unless ($use_hheight =~ /^((\d+)|(\d+%))$/) {
        &main::my_msg("Not allowed value in $COMMAND for hheight : $use_hheight\n");
        $use_hheight="100%";
      }
    } else {
      $use_hheight="100%";
    }
  
    # Wenn keine HTML-Unterschrift, dann nehme die LaTeX-Unterschrift
    unless ($hsign_flag) { $use_hsign=$use_sign; }
  
    if (($use_imglink)&&($use_alt_name)) {
      &main::my_msg("Flags \'embed\' and \'imglink\' in $COMMAND. Ignore \'embed\'.\n");
      $use_embed="";
    }
    if (($codelink_flag)&&(!($use_codelink))) { $use_codelink = "Source"; }
  }
  
  # Prueft, ob die Konvertierungsroutine noetig ist
  sub ok_to_work {
    my $ntc_key;
    my $mod_time;
    
    $ntc_key="$noticy_key";
    $mod_time="$mtime";
  
    if ($main::MAKE_ALL) {return 1;}
    if ($main::NOT_NTCFILE) {return 0;}
    if (((exists $main::NOTICE_OLD{$ntc_key})
      and($main::NOTICE_OLD{$ntc_key} == $mod_time))
      or (exists $main::NOTICE_NEW{$ntc_key})) {return 0;} 
    return 1;
  }
  
  # Diese Subroutine konvertiert Master-Kode in HTML-Kode        #
  sub html_work {
    my ($new_name);
    my ($font_begin,$font_end); 
  
    if ($main::HTML_SIGN_FONT) {
      $font_begin = "<$main::HTML_SIGN_FONT>";
      $font_end = "<\/$main::HTML_SIGN_FONT>";
    }
    if ($use_hwidth) { $width="WIDTH=$use_hwidth"; }
  
    SWITCH: {
      if ($use_type eq "wrl") {$new_name = &wrl_to_html; last SWITCH;}
      &main::my_exit("Unkrown expansion '.$use_type'\n");
    }
    &main::write_to_texfile(
      "% *** Master-Generator $COMMAND fuer HTML ***\n".
      "% BEGIN : $use_local_name.$use_type\n".
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\\end{rawhtml}\n"
    );
    &main::write_to_texfile("<A NAME=$use_label>\n") if ($use_label);
    if ($use_embed) {
      $use_hsign = &main::sign_to_html($use_hsign);
      &main::write_to_texfile("\\begin{rawhtml}\n");
      &main::write_to_texfile("\t<P>\n");
      &main::write_to_texfile("\t<CENTER>\n") if ($use_center);
      &main::write_to_texfile(
        "\t\t<EMBED SRC=\"$new_name\" WIDTH=$use_hwidth HEIGHT=$use_hheight $use_options>\n". 
        "\t\t<BR>$font_begin $use_hsign $font_end\n"
      );
      &main::write_to_texfile("\t</CENTER>\n") if ($use_center);
      &main::write_to_texfile("\\end{rawhtml}\n");
    } elsif (($use_imglink)&&($use_alt_name)) {
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<A HREF=\"$new_name\">\n".
        "\\end{rawhtml}\n"
      );
      push @TO_IMAGE,"NO_BOT_SPACE"; 
      unshift @TO_IMAGE,"$COMMAND";
      &main::image_work(@TO_IMAGE);
      &main::write_to_texfile(
        "\\begin{rawhtml}\n".
        "\t<\/A>\n".
        "\t$font_begin\n".
        "\\end{rawhtml}\n"
      );
        &main::write_to_texfile("\\begin{center}\n") if ($use_center);
        if ($hsign_flag) {
          &main::write_to_texfile("$use_hsign\n");
        } else {
          #$use_sign=&main::sign_to_html($use_sign);
          &main::write_to_texfile("$use_sign\n");
        }
        &main::write_to_texfile("\\end{center}\n") if ($use_center);
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t$font_end\n".
          "\\end{rawhtml}\n"
        );
    } else {
      $use_hsign = &main::sign_to_html($use_hsign);
      &main::write_to_texfile("<A HREF=\"$new_name\">$use_hsign<\/A>\n"); 
    }
    &main::write_to_texfile("<\/A>\n") if ($use_label);
    &main::write_to_texfile(
      "\\begin{rawhtml}\n".
      "\t<BR>\n".
      "\t<P>\n".
      "\\end{rawhtml}\n"
    );
  
    if ($use_codelink) {
      if (-f "$main::WORK_DIR/$main::OUTPUT_DIR/Sources/code-$use_local_name-$use_type.html") {
        &main::write_to_texfile(
          "\\begin{rawhtml}\n".
          "\t<A HREF = \"Sources/code-$use_local_name-$use_type.html\"> $use_codelink </A>\n".
          "\t<P>\n".
          "\\end{rawhtml}\n"
        );
      }
    }
    &main::write_to_texfile("% END : $use_name fuer HTML\n");
  }
  
  sub wrl_to_html {
    my ($DIR_TARGET,$NW_NAME);
    
    $DIR_TARGET="$main::OUTPUT_DIR/VRML/";
    $NW_NAME="VRML/$use_local_name.$use_type";
    
    if (($main::COPY_SOURCE)&&((&ok_to_work)||(!(-f "$main::WORK_DIR/$main::OUTPUT_DIR/$NW_NAME")))) {
      &main::get_underdir("VRML");
      &main::my_print("Copy $use_name to $main::OUTPUT_DIR/$NW_NAME\n");
      &main::copy($use_name,"$main::WORK_DIR/$main::OUTPUT_DIR/$NW_NAME");
      &set_vrml_source($use_name);
      foreach $source (@vrml_source) {
        if (-f "$source") {
          &main::my_print("Copy $source to $DIR_TARGET\n");
          $status=&main::copy($source,"$main::WORK_DIR/$DIR_TARGET");
        } else {
          &main::my_msg("Warning : Source  $source in $use_name is not found\n");
        }
      }
      
      # es hat sich was geaendert 
      $main::SOURCE_IS_CHANGE=1;
    }
     if (($use_codelink)&&((&ok_to_work)||
         (!(-f "$main::WORK_DIR/$main::OUTPUT_DIR/Sources/code-$use_local_name-$use_type.html")))) {
      &generate_html_of_source($use_name);
    }
    if ($main::COPY_SOURCE) { return $NW_NAME;}
    else {
      if ($main::RELATIVE_PFAD) {return "../".$use_relativ_name;}
      else { return $use_name; }
    }
  }
  
  
  # Diese Routine setzt den @vrml_source
  sub set_vrml_source {
    my ($inputfile) = shift;          #Dateiname als Argument
    my ($DESKRIPTOR) = "INPUT_$inputfile";  #Daskriptor generieren
    my ($source);
    my ($line);
  
    &main::my_print("read VRML-file : $inputfile ...\n");
    open ($DESKRIPTOR,$inputfile) or &main::my_exit("Error by open : $inputfile\n");
    while ($line=<$DESKRIPTOR>) {     #lese die Datei Zeilenweise
      chomp($line);                       #entferne Zeilenumbruch
      if ($line =~ /^(\s*)url(\s*)\"(\s*)(.*)(\s*)\"/) {
        $source="$use_dir/$4";
        #if ($5 eq "wrl") {&set_vrml_source("$use_dir/$source");}
        push @vrml_source, $source;
      }
    }
  }
  
  # Generiert HTML-Datei mit Code von VRML-Datei
  sub generate_html_of_source {
    my ($file) = shift;          #Dateiname als Argument
    my $TARGET="Sources";
    my $line;
    
    unless (-f $file) {
      &main::my_msg("Generate of \'codelink\' failed.");
      return;
    }
    unless (-d "$main::WORK_DIR/$main::OUTPUT_DIR/$TARGET") {
      &main::get_underdir("$TARGET");
    }
  
    open (CODE,"$file") or &main::my_exit("Error by open : $file\n");
    &main::my_print("Generate of $main::OUTPUT_DIR/$TARGET/code-$use_local_name-$use_type.html ...\n") if ($main::DEBUG);
    open (HTML,"> $main::WORK_DIR/$main::OUTPUT_DIR/$TARGET/code-$use_local_name-$use_type.html") 
      or &main::my_exit("Error by write : $main::OUTPUT_DIR/$TARGET/code-$use_local_name-$use_type.html\n");
    print HTML "<!Diese Datei wurde von Master-Generator erzeugt>\n";
    print HTML "<HTML>\n";
    print HTML "<HEAD>\n";
    print HTML "<TITLE> Source of $use_local_name.$use_type </TITLE>\n";
    print HTML "</HEAD>\n";
    print HTML "<BODY BGCOLOR=$main::BGCOLOR TEXT=$main::TEXTCOLOR BACKGROUND=$main::BACKGROUND>\n";
    print HTML "<PRE>\n";
    while ($line=<CODE>) { 
      $line =~ s/\t/      /g;      # ersetze die Tabs durch die Leerzeichen
      print HTML "$line"; 
    }
    print HTML "</PRE>\n";
    print HTML "</BODY>\n";
    print HTML "</HTML>\n";
  
    close HTML;
    close CODE;
      
    # es hat sich was geaendert 
    $main::SOURCE_IS_CHANGE=1;
  }
  
}

$MANAGEFUNKTION->{VRML} = \&vrml_work;

1;
